import click
import configparser
from rich.console import Console
import os

from .alerts import get_alerts
from .intro import intro
from .useragent import check_user_agent, set_user_agent, purge_user_agent, _get_useragent_info

# This is the common entry point, and is called when you type richwx into the console.
@click.group()
@click.pass_context
def cli(ctx):
    """A fun little CLI tool that utilizes the National Weather Service API to display
    weather data in the terminal."""
    
    # This will r/w the .ini file, which stores the useragent information needed for the
    # NWS API.
    configs = configparser.ConfigParser()
    path_to_ini_file = os.path.join(os.path.abspath("."), 'useragent.ini')
    
    ctx.obj = {'console' : Console(),
                'config' : configs,
                'user_agent' : _get_useragent_info(configs, path_to_ini_file),
                'path_to_ini_file' : path_to_ini_file
                }

# Add the commands here.
cli.add_command(intro)
cli.add_command(get_alerts)
cli.add_command(set_user_agent)
cli.add_command(purge_user_agent)
cli.add_command(check_user_agent)

if __name__ == '__main__':
    cli()


from rich.table import Table
import click

@click.command()
@click.pass_context
def intro(ctx):
    """Provides an introduction to ensure that the CLI works.
    """
    console = ctx.obj['console']
    console.print()
    
    title = ':tornado: :high_voltage::snowflake: [bold italic][red]Welcome[/] [green]to[/] \
[blue]RichWx[/][/] :wind_blowing_face: :sun_behind_rain_cloud: :thermometer:'
    
    content = "\nThis package utilizes Rich, NWSAPy, and Click to provide a way to get \
National Weather Service data through the CLI!\n\nThis is designed to be a for fun project. There are no plans \
on developing this further, but if there's enough community support, it will be considered.\n"

    console.rule(title)
    console.print(content)
    console.print()

    alerts_sample_table = Table(title = '== SAMPLE ALERTS, NOT REAL ==\n', show_lines= True, show_edge=False)
    
    alerts_sample_table.add_column("Alert Type", justify="center")
    alerts_sample_table.add_column("Location", justify = 'center')
    alerts_sample_table.add_column("Issued", justify="center")
    alerts_sample_table.add_column("Expires", justify = 'center')
    alerts_sample_table.add_column("Sender", justify = 'center')

    alerts_sample_table.add_row(":water_wave:[green]Flood Warning (FAKE)[/]", "Jefferson, Cliff", "1/1/2021 11:19AM EST", "1/1/2021 5:30PM EST", "It's a mystery...")
    alerts_sample_table.add_row(":cloud_with_tornado: [red]Tornado (FAKE)[/]", "Alachua", "1/1/2021 11:19AM EST", "1/1/2021 5:30PM EST", "Another mystery.")
    alerts_sample_table.add_row(":hamburger: Intern Roy missed slider happy hour ", "Nearby Arby's", "12/16/2021 4:00PM", "When he doesn't miss slider happy hour", "NWS Podunk")
    
    console.rule("Sample Alert Table")
    console.print("\nBy running `richwx alerts`, you'll get a table with the alerts based upon the area (state) you pass in. This is what a table would look like:")
    console.print(alerts_sample_table)
    
    # put some padding at the bottom.
    console.print()


import click
import time
from rich.progress import Progress
from rich.table import Table
from nwsapy import api_connector
from nwsapy.services.validation import valid_areas
from datetime import datetime
from .useragent import check_if_user_agent_is_set
    
def make_emoji_string(nws_warning):
    d = {'Red Flag Warning' : ':triangular_flag:\n[bright_magenta]Red Flag Warning',
         'Fire Weather Watch' : ':fire: :eyes:\n[misty_rose3]Fire Weather Watch',
         'Tornado Watch' : ':eyes: :tornado:\n[red]Tornado Watch',
         'Tornado Warning' : ':exclamation_mark: :tornado:\n[red bold]Tornado Warning',
         'Severe Thunderstorm Watch' : ':eyes: :high_voltage [yellow]Severe Thunderstorm Watch',
         'Severe Thunderstorm Warning' : ':exclamation_mark: :high_voltage:\n[yellow bold]Severe Thunderstorm Warning',
         'Hurricane Watch' : ':eyes: :cyclone: :water_wave:\n[magenta]Hurricane Watch',
         'Hurricane Warning' : ':exclamation_mark: :cyclone: :water_wave:\n[red bold]Hurricane Warning',
         'Tropical Storm Watch' : ':eyes: :cyclone:\n[blue]Tropical Storm Watch',
         'Tropical Storm Warning' : ':exclamation_mark: :cyclone:\n[blue bold]Tropical Storm Warning',
         'Flash Flood Watch' : ':eyes: :water_wave:\n[green]Flash Flood Watch',
         'Flash Flood Warning' : ':exclamation_mark: :water_wave:\n[green bold]Flash Flood Warning',
         'Rip Current Statement' : ':water_wave:\n[turquoise2]Rip Current Statement',
         'Freeze Watch' : ':cold_face: :eyes:\n[cyan]Freeze Watch',
         'Freeze Warning' : ':cold_face: :exclamation_mark:\n[blue1]Freeze Warning',
         'High Wind Watch' : ':wind_blowing_face: :eyes:\n[dark_goldenrod]High Wind Watch'
         }

    if nws_warning not in d:
        return nws_warning
    
    return d[nws_warning]

@click.command('alerts')
@click.argument('state', nargs = 1)
@click.pass_context
def get_alerts(ctx, state):
    """Displays a list of NWS alerts."""

    console = ctx.obj['console']
    
    n_steps = 4
    steps_taken = 0

    with Progress() as progress:
        
        # Add a new task bar.
        task = progress.add_task('Validating input...', total = n_steps)
        if len(state) != 2:
            console.print("\n=> [red bold]Attention:[/] You must have the state in its 2 letter abbreviation (ex: FL, MA, etc).\n")
            progress.update(task, advance = steps_taken, description = f"Unsuccessful data validation. See error above.")
            return
        
        state = state.upper()
        if state not in valid_areas():
            console.print(f"\n=> [red bold]Attention:[/] {state} is not a valid state.\n")
            progress.update(task, advance = steps_taken, description = f"Unsuccessful data validation. See error above.")
            return
        
        steps_taken += 1
        
        # validation happened, checking user agent.
        progress.update(task, advance = steps_taken, description = 'Checking User Agent...')
        check_if_user_agent_is_set(ctx)
        user_agent = ctx.obj['user_agent']
        api_connector.set_user_agent(user_agent['app_name'], user_agent['contact'])
        steps_taken += 1
        
        # User agent OK and set. Fetch the data.
        progress.update(task, advance = steps_taken, description = f"Getting alerts for {state}...")
        data = api_connector.get_active_alerts(area = state).to_dict()
        steps_taken += 1
        
        progress.update(task, advance = steps_taken, description = f"Creating table...")
        table = Table(title=f"Alerts for {state}, requested: {str(datetime.utcnow())} UTC", show_lines= True)
        table.add_column("Alert Type", justify="center")
        table.add_column("Location", justify = 'center')
        table.add_column("Issued", justify="center")
        table.add_column("Expires", justify = 'center')
        table.add_column("Sender", justify = 'center')
        
        for key_number in data.keys():
            counties = data[key_number]['areaDesc'].replace(';', ',')
            table.add_row(make_emoji_string(data[key_number]['event']), counties, f"[magenta]{str(data[key_number]['sent'])}", f"[yellow]{str(data[key_number]['expires'])}", data[key_number]['sender_name'])
        steps_taken += 1
    
        progress.update(task, advance = steps_taken, description = f"Completed!")
    
    # need 2 additional print statements here because the bar inteferes with it.
    console.print()
    console.print()
    console.print(table)


import re
import click
import os

def _get_default_ini_values():
    """Returns default ini file values.

    Returns:
        dict: Information for default user agent ini files.
    """
    return {'applicationname' : 'RichWxTerminal',
            'contactinfo' : 'NoneSet'}

def _ini_file_does_not_exist(path):
    """Determines if the ini file does NOT exist.

    Args:
        path (string): The path to the ini file.

    Returns:
        bool: True if the file does exist, false otherwise.
    """
    if os.path.isfile(path):
        return False
    return True

def _get_useragent_info(configs, path_to_ini_file, return_if_new_file_was_created = False):
    """Gets the information from the user agent ini file.

    Args:
        config (configparser.ConfigParser): Used for I/O for the ini file.
        path_to_ini_file (str): The path to the ini file.
        return_if_new_file_was_created (bool, optional): Only checks to see if a new ini file was made. Defaults to False.

    Returns:
        bool/dict: dictionary containing the user agent info, otherwise true/false.
    """

    # in case it's not created, make a file.
    created_new_ini_file = False
    if _ini_file_does_not_exist(path_to_ini_file):
        with open(path_to_ini_file, 'w') as configfile:
            configs['UserAgent'] = _get_default_ini_values()
            configs.write(configfile)
            created_new_ini_file = True
    
    # If we only want to see if we created a new file with default info or not.
    if return_if_new_file_was_created:
        return created_new_ini_file
    
    # Get the user agent information from this ini file.
    configs.read(path_to_ini_file)
    user_agent = {'app_name' : configs['UserAgent']['applicationname'], 'contact' : configs['UserAgent']['contactinfo']}
    return user_agent

def check_if_user_agent_is_set(ctx):
    """Determines if the user agent is set. If not, then it will not show anything on the console.

    Args:
        config (configparser.ConfigParser): Used for parsing the ini file.
        path (str): Path to the ini file.
        rich_console (rich.console.Console): The rich console to print an alert.
    """
    config = ctx.obj['config']
    path = ctx.obj['path_to_ini_file']
    rich_console = ctx.obj['console']
    
    user_agent_info = _get_useragent_info(config, path)
    if user_agent_info['contact'] == "NoneSet":
        rich_console.print("=> [bold red]Attention:[/bold red] User Agent is not set. Be sure to set it running [underline]`richwx set-user-agent`[/underline].")

@click.command()
@click.argument('contact', required = True, nargs = -1, type = str)
@click.pass_context
def set_user_agent(ctx, contact):
    """Sets metadata required by the NWS API.
    This application does NOT transmit this information except to the API.
    When setting the application name and contact, be sure it is something
    that you are okay with releasing publicly in the event of a security breach.
    
    There is functionality to clear this set information by running
    `richwx clear-user-agent`. This resets the information to defaults.
    
    => `contact` should be a website or email; it's a way to for the
    maintainers to contact you in the event of a security breach.
    """
    
    if isinstance(contact, tuple):
        contact = " ".join(contact)
    
    # extract the information from context.
    config = ctx.obj['config']
    path_to_ini = ctx.obj['path_to_ini_file']
    
    # see if the ini file exists. If not, then create it.
    created_new_ini_file = _get_useragent_info(config, path_to_ini, return_if_new_file_was_created=True)
    
    # Read the ini file and set the new contact info.
    config.read(path_to_ini)
    config['UserAgent']['ContactInfo'] = contact
    with open(path_to_ini, 'w') as configfile:
        config.write(configfile)
    
    # Do console printing.
    console = ctx.obj['console']
    if created_new_ini_file:
        console.print(f"\n[red]Had to create new ini file.[/red]\n\tLocation: [underline green]{path_to_ini}[/underline green]")
        
    console.print(f"\nUser agent contact info now set to: [blue]{contact}[/blue]")
    console.print(f"\tReminder: Only the NWS API maintainers will see this information!\n")

@click.command()
@click.pass_context
def purge_user_agent(ctx):
    """Purges the user agent information locally.
    """
    
    config = ctx.obj['config']
    path_to_ini = ctx.obj['path_to_ini_file']
    console = ctx.obj['console']
    
    config.read(path_to_ini)
    config['UserAgent'] = _get_default_ini_values()
    with open(path_to_ini, 'w') as configfile:
        config.write(configfile)
        
    console.print(f"\n[green]PURGED[/green] user agent information locally.\n")

@click.command()
@click.pass_context
def check_user_agent(ctx):
    """Checks the current user agent information.
    """
    
    user_agent = ctx.obj['user_agent']
    console = ctx.obj['console']
    contact = user_agent['contact'] 
    
    if contact == "NoneSet":
        console.print("\n=> [magenta underline]You have not set this value.[/] Set it through [reverse]richwx set-user-agent \[contact]\n")
    else:
        console.print(f"\nContact Information for user agent: [magenta]{contact}\n")


from setuptools import setup, find_packages

version = '0.0.1'

setup(
        name = 'richwx',
        version = version,
        packages = find_packages(),
        entry_points = {
                'console_scripts': [
                'richwx = cli.cli:cli', # cli_folder.cli_file:cli_function
                ],
        },
        install_requires = [
                'Click',
                'rich',
                'nwsapy'
        ]
     )